package de.akquinet.zalit.client;

import java.util.Date;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FlexTable;

public class ProjectDateTable extends FlexTable {
  private final Date _start, _end;
  private final int _initialRows;

  private final Element head;
  private final Element headerTr;

  public ProjectDateTable(final Date start, final Date end, final int initialRows) {
    _start = start;
    _end = end;
    _initialRows = initialRows;
    setBorderWidth(1);

    head = DOM.createTHead();
    headerTr = DOM.createTR();
    DOM.insertChild(this.getElement(), head, 0);
    DOM.insertChild(head, headerTr, 0);
    final Element tBody = getBodyElement();
    DOM.setElementAttribute(tBody, "style", "overflow:auto;text-align: left;");
    DOM.setElementAttribute(head, "style", "text-align: left;color: gray;background-color: blue;");

    getRowFormatter().addStyleName(0, "tableHeader");
    setCellPadding(3);
  }

  @Override
  public void setHeight(final String height) {
    DOM.setElementAttribute(getBodyElement(), "height", height);
  }

  public void setHeader(final int column, final String text) {
    prepareHeader(column);
    if (text != null) {
      DOM.setInnerText(DOM.getChild(headerTr, column), text);
    }
  }

  private void prepareHeader(final int column) {
    if (column < 0) {
      throw new IndexOutOfBoundsException("Cannot create a column with a negative index: " + column);
    }
    final int cellCount = DOM.getChildCount(headerTr);
    final int required = column + 1 - cellCount;
    if (required > 0) {
      addCells(head, 0, required);
    }
  }

  public void init() {
    setText(0, 0, "Projekt");

    final Date date = _start;
    int columns = 1;

    while (date.before(_end)) {
      setHeader(columns, format(date));
      date.setDate(date.getDate() + 1); // Monatsgrenzen...
      columns++;
    }

    for (int row = 1; row < _initialRows + 1; row++) {
      for (int column = 1; column < columns; column++) {
        setWidget(row, column, new Button("feld"));
      }
    }
  }

  private native void addCells(Element table, int row, int num)/*-{
    var rowElem = table.rows[row]; 
    for(var i = 0; i < num; i++){ 
    var cell = $doc.createElement("td"); 
    rowElem.appendChild(cell);   
    }
  }-*/;

  @SuppressWarnings("deprecation")
  private String format(final Date date) {
    return fillup2Digits(date.getDate()) + "." + fillup2Digits(date.getMonth() + 1);
  }

  private static String fillup2Digits(final int date) {
    final String result = String.valueOf(date);
    return (result.length() < 2) ? "0" + result : result;
  }
}
