package de.akquinet.zalit.client;

import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.jboss.errai.bus.client.api.Message;
import org.jboss.errai.bus.client.api.MessageCallback;
import org.jboss.errai.bus.client.api.RemoteCallback;
import org.jboss.errai.bus.client.api.base.MessageBuilder;
import org.jboss.errai.bus.client.framework.ClientMessageBus;
import org.jboss.errai.bus.client.framework.MessageBus;
import org.jboss.errai.bus.client.protocols.MessageParts;
import org.jboss.errai.ioc.client.api.EntryPoint;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * Main application entry point.
 */
@EntryPoint
public class ZalitMainApplication extends VerticalPanel {
  private final ClientMessageBus _messageBus;

  @Inject
  public ZalitMainApplication(final MessageBus bus) {
    _messageBus = (ClientMessageBus) bus;
  }

  @PostConstruct
  public void init() {
    final ProjectDateTable projectDateTable = new ProjectDateTable(5);
    final Button sayHello = new Button("Say Hello!");
    final Label label = new Label();

    /**
     * Register a this as a listener
     */
    _messageBus.subscribe("HelloWorldClient", new MessageCallback() {
      public void callback(final Message message) {
        label.setText(message.get(String.class, "response"));
      }
    });

    /**
     * Register click handler.
     */
    sayHello.addClickHandler(new ClickHandler() {
      public void onClick(final ClickEvent event) {
        MessageBuilder.createMessage().toSubject("ZalitMitarbeiterService").with(MessageParts.ReplyTo, "HelloWorldClient")
            .with("payload", "Hi there!").done().sendNowWith(_messageBus);
      }
    });

    add(sayHello);
    add(label);

    _messageBus.addPostInitTask(new Runnable() {
      public void run() {
        MessageBuilder.createCall(new RemoteCallback<List<String>>() {
          public void callback(final List<String> dates) {
            projectDateTable.init(dates);
          }
        }, DateUtilService.class).getDates("02.03.2010", "10.03.2010");
      }
    });

    add(projectDateTable);

    RootPanel.get().add(this);
  }
}
